﻿Imports System.IO
Imports System.Text
Imports System.Threading
Imports System.Collections.Generic

Module Module_DataLogger
    Private LastValue(1000) As Single
    Friend FieldSeparator As String = ";"
    Friend GCI As Globalization.CultureInfo = Globalization.CultureInfo.InvariantCulture
    Friend FileName As String
    Friend AddDate As Boolean
    Friend AddTime As Boolean
    Friend AddJulianDate As Boolean
    Friend LogLine As String

    Friend Divider As Int32 = 1
    Friend FirstSlot As Int32
    Friend NumSlots As Int32

    Friend DecimalsString As String = "0.000"

    Private logBuffer As New List(Of String)()
    Private logBufferLock As New Object()
    Public bufferSize As Single '= 1000  ' Dimensione del buffer
    Public MaxTime As Integer
    Private writeThread As Thread
    Private stopWriteThread As Boolean = False

    Friend Sub BackupLogFile()
        Dim BackupFname As String = ""
        BackupFname += Application.StartupPath + "\LOGS\" + IO.Path.GetFileNameWithoutExtension(FileName) + "__"
        BackupFname += IO.File.GetCreationTime(FileName).ToString("yyyy_MM_dd__HH_mm_ss", GCI)
        BackupFname += IO.Path.GetExtension(FileName)
        IO.File.Copy(FileName, BackupFname, True)
    End Sub

    Friend Sub DataLogger_Tick()
        ' ----------------------------------------------------- Create log-line
        Dim LogLine As New StringBuilder()
        Dim d As Date = Date.Now

        ' ----------------------------------------------------- Add Date and Time
        If AddDate And AddTime Then
            LogLine.Append(d.ToString("yyyy/MM/dd HH:mm:ss.fff", GCI) + FieldSeparator)
        ElseIf AddDate Then
            LogLine.Append(d.ToString("yyyy/MM/dd", GCI) + FieldSeparator)
        ElseIf AddTime Then
            LogLine.Append(d.ToString("HH:mm:ss.fff", GCI) + FieldSeparator)
        End If

        ' ----------------------------------------------------- Add OA date or Julian date
        If AddJulianDate Then
            'LogLine.Append(DateToOADate(d) + FieldSeparator)
            LogLine.Append(DateToJulianDate(d) + FieldSeparator)
        End If

        ' ----------------------------------------------------- Add log fields
        For i As Int32 = FirstSlot To FirstSlot + NumSlots - 1
            'SMOOTHING ON
            LastValue(i) += (Slots.ReadSlot(i) / Divider - LastValue(i)) * 0.9F
            LogLine.Append((LastValue(i)).ToString(DecimalsString, GCI))
            LogLine.Append(FieldSeparator)
            'SMOOTHING OFF
            'LogLine.Append((Slots.ReadSlot(i) / Divider).ToString(DecimalsString, GCI))
            'LogLine.Append(FieldSeparator)
        Next

        ' ----------------------------------------------------- Remove the last separator
        If LogLine.Length > 0 Then
            LogLine.Length -= FieldSeparator.Length
        End If

        ' Add the log-line to buffer
        SyncLock logBufferLock
            logBuffer.Add(LogLine.ToString())
            If logBuffer.Count >= bufferSize Then
                ' --------------------------------------------- Start the writing thread if not running
                If writeThread Is Nothing OrElse Not writeThread.IsAlive Then
                    stopWriteThread = False
                    writeThread = New Thread(AddressOf WriteLogBuffer)
                    writeThread.Start()
                End If
            End If
        End SyncLock
    End Sub
    Private Sub WriteLogBuffer()
        Dim linesToWrite As List(Of String)
        SyncLock logBufferLock
            linesToWrite = New List(Of String)(logBuffer)
            logBuffer.Clear()
        End SyncLock

        Try
            Using file As New StreamWriter(FileName, True, Encoding.ASCII)
                For Each line As String In linesToWrite
                    file.WriteLine(line)
                Next
            End Using
        Catch ex As Exception
            'Console.WriteLine("Error writing log file: " & ex.Message)
        End Try

        ' ------------------------------------------------ Signal the thread finished
        stopWriteThread = True
    End Sub
    ' =======================================================================
    '  Date to OLE Automation date
    ' =======================================================================
    Public Function DateToOADate(ByVal dt As Date) As String
        Return dt.ToOADate.ToString("0.00000000", GCI)
    End Function

    ' =======================================================================
    '  Date to true Julian date
    ' =======================================================================
    '  OADate = number of days since midnight on the 31st December 1899
    '  Julian = number of days since midday on the 1st of January, 4713 BC
    '  2415018.5 = difference between the two dates
    ' =======================================================================
    Public Function DateToJulianDate(ByVal d As Date) As String
        Return (d.ToOADate + 2415018.5).ToString("0.00000000", GCI)
    End Function

End Module
